#!/bin/bash

set -e

setUp() {
    JFMC_VERSION="%%myVersion%%"

    JFI_INSIGHT_SERVER_VERSION=${JFMC_VERSION}
    JFI_SCHEDULER_VERSION=${JFMC_VERSION}
    JFI_EXECUTOR_VERSION=${JFMC_VERSION}

    DOCKER_SERVER_CURRENT_VER=$(docker version --format '{{.Server.Version}}') || DOCKER_SERVER_CURRENT_VER=""

    # Mission Control installer workdir
    JFMC_DIR="${HOME}/.jfrog/jfmc/jfmc-installer"

    # Mission Control installer conf file, should be created in the first run
    JFMC_CONF_FILE_NAME="jfmc-env.conf"
    JFMC_CONF="${JFMC_DIR}/${JFMC_CONF_FILE_NAME}"
    JFMC_MOUNT_ROOT_DEFAULT="${HOME}/.jfrog/jfmc"
    JFMC_PORT="8080"

    if [ ! -d "${JFMC_DIR}" ]; then
        mkdir -p "${JFMC_DIR}"
    fi
}

loadInstallerConfigFile() {
    # Load the installer configuration file
    if [ -f "$JFMC_CONF" ]; then
        source "${JFMC_CONF}"
    else
        echo "ERROR: configuration file does not exist [${JFMC_CONF}]"
        exit 1
    fi

    if [ -z "${JFMC_MOUNT_ROOT}" ]
    then
        echo "ERROR: JFMC_MOUNT_ROOT is not set, please update the conf file [${JFMC_CONF}]"
        exit 1
    fi
    echo "INFO: Using JFMC_MOUNT_ROOT=${JFMC_MOUNT_ROOT}"

    ETC_FILE=${JFMC_MOUNT_ROOT}/jfmc/etc/mission-control.properties
}

setupDockerParameters() {
    # Pass an overwritten port
    if [ -f ${ETC_FILE} ]; then
        if [ $(grep "^server.port" ${ETC_FILE}) ]; then
            JFMC_PORT=$(grep  "^server.port" ${ETC_FILE} | awk -F'=' '{print $2}')
        fi
    fi

    # Setup options for connecting to docker host
    if [ -z "${DOCKER_HOST}" ]; then
        DOCKER_HOST="/var/run/docker.sock"
    fi
    if [ -S "${DOCKER_HOST}" ]; then
        DOCKER_ADDR="-v ${DOCKER_HOST}:${DOCKER_HOST} -e DOCKER_HOST"
    else
        DOCKER_ADDR="-e DOCKER_HOST -e DOCKER_TLS_VERIFY -e DOCKER_CERT_PATH"
    fi

    # Only allocate tty if we detect one
    if [ -t 1 ]; then
        DOCKER_RUN_OPTIONS="-t"
    fi
    if [ -t 0 ]; then
        DOCKER_RUN_OPTIONS="${DOCKER_RUN_OPTIONS} -i"
    fi

    JFMC_VOLUMES="-v ${JFMC_DIR}:/data -v ${JFMC_MOUNT_ROOT}:/jfmc_global_mount_root"

    JFMC_DOCKER_REPO="%%dockerRepo%%"

    IMAGE="${JFMC_DOCKER_REPO}/jfrog/jfmc-installer:${JFMC_VERSION}"

    [ "${JFMC_PORT}" != "8080" ] && echo "INFO: Using none default http port: ${JFMC_PORT}" || true
}

credsChangeMessage() {
    echo -e """Please set JFMC_DOCKER_REPO_LOGIN and JFMC_DOCKER_REPO_PASSWORD as environment variables, script has been modified to not consider credentials passed during its invocation.

export JFMC_DOCKER_REPO_LOGIN=<username>
export JFMC_DOCKER_REPO_PASSWORD=<token>

The above lines can be used to set env variables for this session. Add them to ~/.bashrc to set it permanently"""
    exit 1
}

createInstallerConfigFile() {
    # Create the installer configuration file
    if [ ! -f "${JFMC_CONF}" ]; then
        echo -e "\nJFMC_MOUNT_ROOT must be set on jfmc first installation \nPlease type desired JFMC_MOUNT_ROOT location"
        if [[ $AUTOMATION_MODE == "true" ]]; then
            echo "INFO: Non-interactive mode is enabled, considering default value - ${JFMC_MOUNT_ROOT_DEFAULT}"
            JFMC_MOUNT_ROOT=${JFMC_MOUNT_ROOT_DEFAULT}
        else
            read -p "Enter mount root [${JFMC_MOUNT_ROOT_DEFAULT}]: " JFMC_MOUNT_ROOT
            JFMC_MOUNT_ROOT=${JFMC_MOUNT_ROOT:-${JFMC_MOUNT_ROOT_DEFAULT}}
        fi
        
        addDataLocToConf
    fi
}

setUpDevDockerParamters(){
    if [[ -z "${JFMC_DOCKER_REPO_LOGIN}" || "${JFMC_DOCKER_REPO_LOGIN}" == "" || -z "${JFMC_DOCKER_REPO_PASSWORD}" || "${JFMC_DOCKER_REPO_PASSWORD}" == "" ]]; then
        credsChangeMessage
    fi

    # Before running, try to login to the registry if login password is provided
    echo "Going to docker login ${JFMC_DOCKER_REPO}...."
    docker login -u ${JFMC_DOCKER_REPO_LOGIN} -p ${JFMC_DOCKER_REPO_PASSWORD} ${JFMC_DOCKER_REPO}
    JFMC_EXTRA_LOGIN_PARAMS="-e JFMC_DOCKER_REPO_LOGIN=${JFMC_DOCKER_REPO_LOGIN} -e JFMC_DOCKER_REPO_PASSWORD=${JFMC_DOCKER_REPO_PASSWORD}"
}

# Set env variables common to dev and release script
setCommonEnvVars() {
  COMMON_ENV_VARS="-e OFFLINE_MODE=${OFFLINE_MODE} -e JFMC_CONF_FILE_NAME=${JFMC_CONF_FILE_NAME} -e JFMC_PORT=${JFMC_PORT} \
  -e JFMC_DOCKER_REPO=${JFMC_DOCKER_REPO} -e JFMC_VERSION=${JFMC_VERSION} \
  ${JFMC_EXTRA_LOGIN_PARAMS} ${JFMC_VOLUMES} ${DOCKER_RUN_OPTIONS} ${DOCKER_ADDR} \
  -e JFI_INSIGHT_SERVER_VERSION=${JFI_INSIGHT_SERVER_VERSION} -e JFI_EXECUTOR_VERSION=${JFI_EXECUTOR_VERSION} \
  -e JFI_SCHEDULER_VERSION=${JFI_SCHEDULER_VERSION} -e JFMC_MOUNT_ROOT=${JFMC_MOUNT_ROOT} -e AUTOMATION_MODE=${AUTOMATION_MODE} \
  -e DOCKER_SERVER_CURRENT_VER=${DOCKER_SERVER_CURRENT_VER}"
}

addDataLocToConf(){
    # Resolve ~ (tilde) with actual HOME of current user
    JFMC_MOUNT_ROOT=$(echo "${JFMC_MOUNT_ROOT}" | sed -e "s#^~#${HOME}#")

    # Write to conf file
    echo "export JFMC_MOUNT_ROOT=${JFMC_MOUNT_ROOT}" >> ${JFMC_CONF}
    echo "JFMC_MOUNT_ROOT is now specified in ${JFMC_CONF}, as ${JFMC_MOUNT_ROOT}"
}

# IMP : leave a blank line at the end of this file
