#!/bin/bash
set -e

function createTempDir
{
  tmpDir=$(mktemp -d 2>/dev/null || mktemp -d -t 'tmpDir')
  echo "INFO: Generating certs in temp location: ${tmpDir}"
}

function rmTempDir
{
  rm ${tmpDir}/*
  rmdir ${tmpDir}
}

function genJfmcKeyStore
{
  keytool -genkeypair -alias secure-jfmc -keyalg RSA \
          -dname "CN=*,OU=JFMC,O=JFrog,L=Toulouse,S=France,C=fr" \
          -keystore ${tmpDir}/jfmc-keystore.jks \
          -storepass ${storePassword} \
          -keypass ${storePassword}

  keytool -exportcert -alias secure-jfmc \
          -file ${tmpDir}/jfmc-public.cer \
          -keystore ${tmpDir}/jfmc-keystore.jks \
          -storepass ${storePassword}

  keytool -importkeystore \
          -srcalias secure-jfmc \
          -srckeystore ${tmpDir}/jfmc-keystore.jks \
          -destkeystore ${tmpDir}/jfmc-keystore.p12 \
          -deststoretype PKCS12 \
          -srckeypass ${storePassword} \
          -srcstorepass ${storePassword} \
          -deststorepass ${storePassword}

  openssl pkcs12 -in ${tmpDir}/jfmc-keystore.p12 \
                 -nokeys \
                 -nodes \
                 -out ${tmpDir}/jfmc.crt \
                 -password pass:${storePassword} \
                 -passin pass:${storePassword}
}

function genInsightKeyStore
{
  keytool -genkeypair -alias secure-insight -keyalg RSA \
          -dname "CN=*,OU=Insight,O=JFrog,L=Bengaluru,S=Kan,C=in" \
          -keystore ${tmpDir}/insight-keystore.jks \
          -storepass ${storePassword} \
          -keypass ${storePassword}

  keytool -exportcert -alias secure-insight \
          -file ${tmpDir}/insight-public.cer \
          -keystore ${tmpDir}/insight-keystore.jks \
          -storepass ${storePassword}

  keytool -importkeystore \
          -srcalias secure-insight \
          -srckeystore ${tmpDir}/insight-keystore.jks \
          -destkeystore ${tmpDir}/insight-keystore.p12 \
          -deststoretype PKCS12 \
          -noprompt \
          -srckeypass ${storePassword} \
          -srcstorepass ${storePassword} \
          -deststorepass ${storePassword}


  openssl pkcs12 -in ${tmpDir}/insight-keystore.p12 \
                 -nocerts \
                 -nodes \
                 -out ${tmpDir}/insight.key \
                 -password pass:${storePassword} \
                 -passin pass:${storePassword}
  openssl pkcs12 -in ${tmpDir}/insight-keystore.p12 \
                 -nokeys \
                 -nodes \
                 -out ${tmpDir}/insight.crt \
                 -password pass:${storePassword} \
                 -passin pass:${storePassword}
}

function importInTrustStore
{
  keytool -importcert -keystore ${tmpDir}/jfmc-truststore.jks \
          -alias insightcert \
          -noprompt \
          -file ${tmpDir}/insight-public.cer \
          -storepass ${storePassword}

  keytool -importcert -keystore ${tmpDir}/insight-truststore.jks \
          -alias jfmccert \
          -noprompt \
          -file ${tmpDir}/jfmc-public.cer \
          -storepass ${storePassword}
}

function overwriteExisting
{
  echo "INFO: Moving certs from temp location: ${tmpDir} to ${jfmcSecurity} and ${insightSecurity}"
  mv -f ${tmpDir}/jfmc-truststore.jks ${jfmcSecurity}/.
  mv -f ${tmpDir}/jfmc-keystore.jks ${jfmcSecurity}/.
  mv -f ${tmpDir}/jfmc.crt ${insightSecurity}/.
  mv -f ${tmpDir}/insight-truststore.jks ${insightSecurity}/.
  mv -f ${tmpDir}/insight-keystore.jks ${insightSecurity}/.
  mv -f ${tmpDir}/insight.key ${insightSecurity}/.
  mv -f ${tmpDir}/insight.crt ${insightSecurity}/.
}

function usage
{
  echo "Usage: genCerts.sh <store_password> [path_of_jfmc_security] [path_of_insight_security]"
  exit 1
}

if [ "$#" -eq 1 ]; then
  current="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
  storePassword=$1
  jfmcSecurity=${current}
  insightSecurity=${current}
elif [ "$#" -eq 3 ]; then
  storePassword=$1
  jfmcSecurity=$2
  insightSecurity=$3
else
  usage
fi

# Check if key generation tools are available
for tool in "keytool" "openssl"; do
  hash $tool 2>/dev/null || { echo >&2 "\"$tool\" is not installed or not available in path, this is needed to genearate keys for secure interservice communication"; exit 1; }
done

createTempDir
genInsightKeyStore
genJfmcKeyStore
importInTrustStore
overwriteExisting
rmTempDir