#!/bin/bash

set -e

#Print the input with additional formatting to indicate a section/title
title () {
    echo
    echo "-----------------------------------------------------"
    printf "| %-50s|\n" "$1"
    echo "-----------------------------------------------------"
}

# This function prints the echo with color.
#If invoked with a single string, treats it as INFO level
#Valid inputs for the second parameter are DEBUG and INFO
log() {
    echo ""
    echo -e $1
    echo ""
}

# Utility method to display warnings and important information
warn() {
    echo ""
    echo -e "\033[33m $1 \033[0m"
    echo ""
}
# A small timeout ensures that the content printed in the console and the logs remains legible
read_timeout=1

# Labels used in messages displayed to the user
ELASTIC_SEARCH_LABEL='Elasticsearch'
MONGO_LABEL='MongoDB'

# PREREQUISITES
DOCKER_SERVER_MIN_VER='1.11'
#DOCKER_SERVER_CURRENT_VER, this should be passed by the installer script [mission-control.sh] as $(docker version --format '{{.Server.Version}}') || DOCKER_SERVER_CURRENT_VER=""

MAX_OPEN_FILES_MIN_VALUE=10000
MAX_OPEN_FILES_REQUIRED_VALUE=32000

# JFMC_APP_SERVICES_LIST, the services names as they are in the docker-cmpose file, SPACE separated
JFMC_APP_SERVICES_LIST="server insight_server"

# PREREQUISITES
DOCKER_SERVER_MIN_VER='1.11'
#DOCKER_SERVER_CURRENT_VER, this should be passed by the installer script [mission-control.sh] as $(docker version --format '{{.Server.Version}}') || DOCKER_SERVER_CURRENT_VER=""

#Repeated Messages
LOCALHOST_WARNING="Important: When answering the prompts that follow, remember to give a URL that will be reachable from inside other docker containers\n\
For instance, specifying 'localhost' will not work. The default value (docker.for.mac.localhost) will work for MacOS"

aparse()
{
    if [ ! -z $1 ]; then
        command=$1
        shift
    else
        command=usage
        return 0
    fi

    if [[ $# > 0 ]] ; then
              ALL_ARGS="${ALL_ARGS}${1} "
    fi
}

if [ -z "$COMPOSE_PROJECT_NAME" ]; then
    COMPOSE_PROJECT_NAME="jfmc"
fi

DEFAULT_INSTALLER_DATA_FOLDER="/data"
 
# Installed version
JFMC_CONF="$DEFAULT_INSTALLER_DATA_FOLDER/$JFMC_CONF_FILE_NAME"
[ -f "$JFMC_CONF" ] && source "$JFMC_CONF" || true
DISPLAY_INSTALL_LOG_LOCATION="$JFMC_MOUNT_ROOT/jfmc-installer"
# create all files (except for jfmc-conf) in data folder
INSTALLER_DATA_FOLDER="/jfmc_global_mount_root/jfmc-installer"
INSTALLED_VERSION_FILE_NAME="version.current"
INSTALLED_VERSION_FILE="${INSTALLER_DATA_FOLDER}/$INSTALLED_VERSION_FILE_NAME"
INSTALLED_VERSION_FILE_HISTORY="${INSTALLER_DATA_FOLDER}/version.history"

# Create install folder if its not avialable
mkdir -p "${INSTALLER_DATA_FOLDER}"

: ${INSTALL_ELASTIC_SEARCH:="Y"}
: ${INSTALL_MONGO:="Y"}

INSTALLED_VERSION=''
JFMC_GLOBAL_MOUNT_ROOT="/jfmc_global_mount_root" # This should not be used, unless special maintenance is needed

loadInstalledVersion(){
    if [ -f $INSTALLED_VERSION_FILE ]
    then
        INSTALLED_VERSION=$(cat $INSTALLED_VERSION_FILE)
    fi
}
loadInstalledVersion

# Maintainer / Installer version
MAINTAINER_VERSION_FILE="/opt/jfrog/jfmc/installer/version.current"
MAINTAINER_VERSION=''
if [ -f $MAINTAINER_VERSION_FILE ]
then
    MAINTAINER_VERSION=$(cat $MAINTAINER_VERSION_FILE)
fi

export IMAGE_MOUNT_ROOT="/var/opt/jfrog/mission-control"
export JFMC_PORT


# Utility method to remove environment variables
# 1st parameter "the key to remove"
removeFromEnvFile(){
    local key_to_remove=$1
    #Delete the original entry. NOTE: This will also ensure that if a value is not available, it is removed from the file
    sed -i "/export $key_to_remove/d" $JFMC_CONF
}

# Utility method to add environment variables
# 1st parameter "the key to add"
addToEnvFile(){
    if [ ! -f $JFMC_CONF ]; then
        return
    fi
    local key_to_add=$1
    # Get the value associated with this variable (indirect reference)
    local value_to_add=${!1}
    
    removeFromEnvFile $key_to_add
    if [[ ! -z "$value_to_add" ]]; then
        #log "Executing echo export $key_to_add=\"$value_to_add\" >> $JFMC_CONF"
        echo "export $key_to_add=\"$value_to_add\"" >> ${JFMC_CONF}
    fi
}

exitOnError()
{
    msg=$1
    echo ""
    echo -e "\033[31m ERROR: ${msg} \033[0m"
    exit 1
}

#Given a file, this function runs the jq command and saves it back
replace_value() {
	file_name=$1
	jq_command=$2

	cat $file_name|jq $jq_command > ~/tmpfile && mv ~/tmpfile $file_name
}

usage() {
    cat ./readme.txt
    exit 1
}

version_lt() { test "$(echo "$@" | tr " " "\n" | sort -rV | head -n 1)" != "$1"; }

# $1 should be WARNING or ERROR
printUlimitErrorMsg()
{
    local error_level=$1
    local number=$2
    MSG="
${error_level}: Max number of open files [${number}] is too low !!!

Add the following to /etc/security/limits.conf file on HOST:
* soft nofile ${MAX_OPEN_FILES_REQUIRED_VALUE}
* hard nofile ${MAX_OPEN_FILES_REQUIRED_VALUE}

Note: Also verify docker limits configuration if exists [usually in /etc/default/docker]

"

    case ${error_level} in
    "ERROR")
        echo -e "\033[31m${MSG}\033[0m "
    ;;
    "WARNING")
        echo -e "\033[33m${MSG}\033[0m"
    ;;
    esac

    echo
}

checkPrerequisites()
{
    #echo "Check dependencies:"

    if [ -z ${DOCKER_SERVER_CURRENT_VER} ] || [ "${DOCKER_SERVER_CURRENT_VER}" == "" ]
    then
        echo "WARNING: Docker daemon version is unknown"
        # Do not exit on warning
    fi

    if version_lt ${DOCKER_SERVER_CURRENT_VER} ${DOCKER_SERVER_MIN_VER}
    then
        echo ""
        echo "ERROR: Mission Control requires Docker daemon version ${DOCKER_SERVER_MIN_VER} or later. The current version is ${DOCKER_SERVER_CURRENT_VER}."
        return 1
    fi

    local max_map_count=$(cat /proc/sys/vm/max_map_count)
    local recommended_map_count=262144
    if [ $max_map_count -lt $recommended_map_count ]; then
        exitOnError  "Not enough virtual memory to run $ELASTIC_SEARCH_LABEL reliably. Recommended: [vm.max_map_count=$recommended_map_count]. \
Please refer to [https://www.elastic.co/guide/en/elasticsearch/reference/current/docker.html#docker-cli-run-prod-mode]"
    fi


    # echo "Check max number of open files:"
    currentMaxOpenFilesVal=$(ulimit -n)

    if [ "${currentMaxOpenFilesVal}" != "unlimited" ] ; then

        # MAX_OPEN_FILES_REQUIRED_VALUE
        if [ ${currentMaxOpenFilesVal} -lt ${MAX_OPEN_FILES_REQUIRED_VALUE} ]
        then
            #echo "Trying to set max open files = ${MAX_OPEN_FILES_REQUIRED_VALUE}"
            ulimit -n ${MAX_OPEN_FILES_REQUIRED_VALUE} 2>/dev/null
            if [ $? -gt 0 ]
            then
                # Cannot set MAX_OPEN_FILES_REQUIRED_VALUE, will try to set MAX_OPEN_FILES_MIN_VALUE
                # MAX_OPEN_FILES_MIN_VALUE
                if [ ${currentMaxOpenFilesVal} -lt ${MAX_OPEN_FILES_MIN_VALUE} ]
                then
                    #echo "Trying to set max open files = ${MAX_OPEN_FILES_MIN_VALUE}"
                    ulimit -n ${MAX_OPEN_FILES_MIN_VALUE} 2>/dev/null
                    if [ $? -gt 0 ]; then
                        printUlimitErrorMsg "ERROR" "${currentMaxOpenFilesVal}"
                        return 1
                    fi
                fi
                printUlimitErrorMsg "WARNING" "${currentMaxOpenFilesVal}"
                # Do not exit on warning
            fi
        fi

    fi

    # dependencies check is passed
    return 0
}

runDockerCommand() {
    local action=$1

    if [[ $OFFLINE_MODE == true && $action == "pull" ]]; then
        return
    fi

    #If a modified docker-compose file has been created, use that instead
    if [ -f ${INSTALLER_DATA_FOLDER}/docker-compose.json ]; then
        cp ${INSTALLER_DATA_FOLDER}/docker-compose.json ./docker-compose.json
    fi

    case $action in
    "pull")
        if [ ! -z "$JFMC_DOCKER_REPO_LOGIN" ]; then
            docker login -u $JFMC_DOCKER_REPO_LOGIN -p $JFMC_DOCKER_REPO_PASSWORD $JFMC_DOCKER_REPO
        fi
    ;;
    "logs")
        action="logs -f"
    ;;
    "stop")
        # Stop Mission Control App layer first, than the DB layer
        docker-compose -f docker-compose.json -p ${COMPOSE_PROJECT_NAME} ${action} ${JFMC_APP_SERVICES_LIST}
    ;;
    "stopApp")
        # Stop Mission Control App layer first, than the DB layer
        docker-compose -f docker-compose.json -p ${COMPOSE_PROJECT_NAME} "stop" ${JFMC_APP_SERVICES_LIST}
        return $?
    ;;
    esac

    docker-compose -f docker-compose.json -p ${COMPOSE_PROJECT_NAME} ${action}
}

# Returned code:
#0) $1=$2
#1) $1>$2
#2) $1<$2
verComp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i
    local ver1=($1)
    local ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

isUpgradeAllowed() {

    local installedVersion=$INSTALLED_VERSION
    local maintainerVersion=$MAINTAINER_VERSION

    if [ -z $MAINTAINER_VERSION ]
    then
        exitOnError "MAINTAINER_VERSION is unknown"
    fi

    # This is a fresh installation
    if [ -z $INSTALLED_VERSION ]
    then
        return 0
    fi

    if [[ "$installedVersion" =~ ^b.* ]] && [[ "$maintainerVersion" =~ ^b.* ]]
    then
        installedVersion=($(echo $INSTALLED_VERSION | sed "s/^b//g"))
        maintainerVersion=($(echo $MAINTAINER_VERSION | sed "s/^b//g"))
    fi

    verComp $installedVersion $maintainerVersion 2>/dev/null
    if [ $? -eq 1 ]
    then
        return 1
    fi

    return 0
}

isCommandAllowed()
{
    local command=$1
    local installedVersion=$INSTALLED_VERSION
    local maintainerVersion=$MAINTAINER_VERSION

    case ${command} in
        "install"|"installLocal")
            [ -f $INSTALLED_VERSION_FILE ] && exitOnError "Mission Control is already installed. To upgrade, use: ./mission-control upgrade"
            checkPrerequisites || exitOnError "Prerequisites are not satisfied"
        ;;
        "upgrade")
            [ -f $INSTALLED_VERSION_FILE ] || exitOnError "Mission Control is not installed. To install, use: ./mission-control install"
            checkPrerequisites || exitOnError "Prerequisites are not satisfied"
            isUpgradeAllowed || exitOnError "Unable to upgrade to an older version"
        ;;
        "version")

        ;;
        "info")

        ;;
        *)
            # Is installed
            [ -f $INSTALLED_VERSION_FILE ] || exitOnError "Mission Control is not installed. To install, use: ./mission-control install"
            # Is correct version
            [ "${installedVersion}" == "${maintainerVersion}" ] || exitOnError "You are using the Mission Control installer version [${maintainerVersion}] which is different from the currently installed version [${installedVersion}]. Consider upgrading your version of Mission Control using: ./mission-control upgrade"
        ;;

    esac

    return 0
}

setVersions()
{
    if [ ! -f ${MAINTAINER_VERSION_FILE} ]
    then
        return 1
    fi

    cat ${MAINTAINER_VERSION_FILE} > ${INSTALLED_VERSION_FILE}
    cat ${MAINTAINER_VERSION_FILE} >> ${INSTALLED_VERSION_FILE_HISTORY}

    echo "${INSTALLED_VERSION_FILE}: $(cat ${INSTALLED_VERSION_FILE})"

    return 0
}

createMongoUsers(){
    docker cp mongodb "${COMPOSE_PROJECT_NAME}_mongodb_1":/ && \
    docker exec -i "${COMPOSE_PROJECT_NAME}_mongodb_1" /mongodb/createMongoUsers.sh
}


createUsers(){
    if [[ $INSTALL_MONGO =~ y|Y ]]; then
        createMongoUsers
    fi
}

executeMigrationIfNeeded()
{
    local migrationName=$1
    local migrationForVersion=$2
    verComp $INSTALLED_VERSION $migrationForVersion 2>/dev/null
    if [ $? -eq 2 ]
    then
        log "\nINFO: Executing migration from [${INSTALLED_VERSION}] to [${migrationForVersion}] - ${migrationName}"
        eval ${migrationName}
    fi
}

# backward compatible issues
# Mission Control should be stopped before this process
backwardCmpatibleProcess()
{
    log "\nINFO: Executing possible migration steps"
    executeMigrationIfNeeded "migrateToSingleMount" "2"
    executeMigrationIfNeeded "genInternalCerts" "2.1.0"
}

migrateToSingleMount()
{
    local JFMC_MOUNT_V1="${JFMC_GLOBAL_MOUNT_ROOT}"
    local JFMC_MOUNT_V2="${JFMC_GLOBAL_MOUNT_ROOT}/jfmc"
    log "\nINFO: Moving data,etc,logs from [${JFMC_MOUNT_V1}] to [${JFMC_MOUNT_V2}]"

    if [ ! -d ${JFMC_MOUNT_V2} ]; then
        mkdir -p ${JFMC_MOUNT_V2} && \
        echo "\nINFO: New folder at [${JFMC_MOUNT_V2}] was created"
    fi

    for folderToMigrate in "logs" "etc" "data"
    do
        local folderToMigrateV1="${JFMC_MOUNT_V1}/${folderToMigrate}"
        local folderToMigrateV2="${JFMC_MOUNT_V2}/${folderToMigrate}"
        if [ -d "${folderToMigrateV1}" ] && [ ! -d "${folderToMigrateV2}" ]
        then
            mv ${folderToMigrateV1} ${folderToMigrateV2} && \
            log "\nINFO: ${folderToMigrate} folder renamed was:[${folderToMigrateV1}] new:[${folderToMigrateV2}]"
        fi
    done
}

genInternalCerts()
{
    log "\nINFO: Generating internal certificates"
    local jfmcEtc=${JFMC_GLOBAL_MOUNT_ROOT}/jfmc/etc
    local jfmcSecurity=${jfmcEtc}/security/
    local insightSecurity=${JFMC_GLOBAL_MOUNT_ROOT}/insight/etc/security/
    for folder in ${jfmcSecurity} ${insightSecurity}
    do
        if [ ! -d ${folder} ]; then
            mkdir -p ${folder}
        fi
    done
    local storePassword=$(cat /dev/urandom | env LC_CTYPE=C tr -cd 'a-f0-9' | head -c 16)
    ./genCerts.sh ${storePassword} ${jfmcSecurity} ${insightSecurity} && \
    echo ${storePassword} > ${jfmcSecurity}/passwd && \
    echo ${storePassword} > ${insightSecurity}/passwd || ( echo -ne "Failed to generate internal certificates" && exit 1 )
}

# Utility method to check if a value (first parameter) exists in an array (2nd parameter)
# 1st parameter "value to find"
# 2nd parameter "The array to search in. Please pass a string with each value separated by a comma"
# Example: containsElement "y" "y Y n N"
containsElement () {
  local searchElement=$1
  local searchArray=($2)
  local found=1
  for elementInIndex in "${searchArray[@]}";do
    if [[ $elementInIndex == $searchElement ]]; then
        found=0
    fi
  done
  return $found
}

# Utility method to get user's choice
# 1st parameter "what to ask the user"
# 2nd parameter "what choices to accept, separated by spaces"
# 3rd parameter "what is the default choice (to use if the user simply presses Enter)"
# Example 'getUserChoice "Are you feeling lucky? Punk!" "y n Y N" "y"'

getUserChoice(){
    local choice="na"
    local text_to_display=$1
    local choices=$2
    local default_choice=$3

    until containsElement "$choice" "$choices"; do
        echo;
        sleep $read_timeout #This ensures correct placement of the question.
        if [[ $AUTOMATION_MODE == "true" ]]; then
            choice=$default_choice
        else
            read -p "$text_to_display:" choice
            : ${choice:=$default_choice}
        fi
    done
    users_choice=$choice
    if [[ -f $INSTALLATION_LOG_FILE ]]; then
        #Ensures the choices made by the user are also put into the log file
        echo "$text_to_display: $users_choice" >> $INSTALLATION_LOG_FILE
    fi
}

# Utility method to get user's input
# 1st parameter "what to ask the user"
# 3rd parameter "what is the default choice (to use if the user simply presses Enter)"
# Example 'getUserInput "Are you feeling lucky? Punk!" "y"'
getUserInput(){
    local choice="na"
    local text_to_display=$1
    local default_choice=$2
    users_choice=
    
    if [ ! -z $default_choice ]; then
        text_to_display="$1 [$default_choice]:"
    else
        text_to_display="$1 :"
    fi

    echo "";echo "";
    sleep $read_timeout #This ensures correct placement of the question.
    read -p "$text_to_display" choice
    if [ ! -z $default_choice ]; then
        : ${choice:=$default_choice}
    fi
    users_choice=$choice
    sleep $read_timeout #This ensures correct logging
}

getElasticSearchDetails() {
    #Variables for Elastic search
    : ${ELASTIC_SEARCH_URL:="http://docker.for.mac.localhost:9200"}
    : ${ELASTIC_SEARCH_USERNAME:=""}
    : ${ELASTIC_SEARCH_PASSWORD:=""}
    #Ask the user the URL to connect to
    
    getUserInput "Please enter the $ELASTIC_SEARCH_LABEL URL" "$ELASTIC_SEARCH_URL"
    ELASTIC_SEARCH_URL=$users_choice
    ELASTIC_SEARCH_WRITE_URL=$users_choice
    ELASTIC_SEARCH_READ_URL=$users_choice

    addToEnvFile "ELASTIC_SEARCH_URL"
    addToEnvFile "ELASTIC_SEARCH_WRITE_URL"
    addToEnvFile "ELASTIC_SEARCH_READ_URL"

    getUserChoice "Does this $ELASTIC_SEARCH_LABEL instance need credentials? (If you choose 'Y', you will be prompted to enter them) [Y/n]" "y n Y N" "y"
    ELASTIC_SEARCH_HAS_CREDENTIALS=${users_choice}
    addToEnvFile "ELASTIC_SEARCH_HAS_CREDENTIALS"

    if [[ $ELASTIC_SEARCH_HAS_CREDENTIALS =~ y|Y ]]; then

        #Ask the user the ID
        getUserInput "Please enter the User ID " "$ELASTIC_SEARCH_USERNAME"
        ELASTIC_SEARCH_USERNAME=$users_choice


        #Ask the user the password
        getUserInput "Please enter the Password " "$ELASTIC_SEARCH_PASSWORD"
        ELASTIC_SEARCH_PASSWORD=$users_choice

    else
        ELASTIC_SEARCH_USERNAME=        
        ELASTIC_SEARCH_PASSWORD=        
    fi
    addToEnvFile "ELASTIC_SEARCH_USERNAME"
    addToEnvFile "ELASTIC_SEARCH_PASSWORD"
}

externalizeElasticSearch() {
    title "$ELASTIC_SEARCH_LABEL"
    log "Install $ELASTIC_SEARCH_LABEL as part of this installation? \
(Choose 'N' if you want to externalize $ELASTIC_SEARCH_LABEL)"
    getUserChoice "Install $ELASTIC_SEARCH_LABEL? [Y/n]" "y n Y N" "y"

    INSTALL_ELASTIC_SEARCH=${users_choice}
    addToEnvFile "INSTALL_ELASTIC_SEARCH"
    
    if [[ $INSTALL_ELASTIC_SEARCH =~ n|N ]]; then
        getElasticSearchDetails
        updateElasticSearchDetails
    else
        removeFromEnvFile "ELASTIC_SEARCH_URL"
        removeFromEnvFile "ELASTIC_SEARCH_USERNAME"
        removeFromEnvFile "ELASTIC_SEARCH_PASSWORD"
    fi  
}

updateElasticSearchDetails() {
    if [[ $INSTALL_ELASTIC_SEARCH =~ n|N ]]; then

        #Remove the elastic search container from the docker-compose file
        cat docker-compose.json|jq 'del(.services.elasticsearch)' > ~/tmpfile && mv ~/tmpfile docker-compose.json
        
        #Remove the entry that makes insight_server depend on elasticsearch container
        cat docker-compose.json|jq 'del(.services.insight_server.depends_on[] | select( . == "elasticsearch") )' > ~/tmpfile && mv ~/tmpfile docker-compose.json

        cat docker-compose.json|jq ".services.insight_server.environment.ELASTIC_SEARCH_URL=\"$ELASTIC_SEARCH_URL\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq ".services.insight_server.environment.ELASTIC_SEARCH_WRITE_URL=\"$ELASTIC_SEARCH_WRITE_URL\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq ".services.insight_server.environment.ELASTIC_SEARCH_READ_URL=\"$ELASTIC_SEARCH_READ_URL\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json


        if [ ! -z $ELASTIC_SEARCH_PASSWORD ]; then
            cat docker-compose.json|jq ".services.insight_server.environment.ELASTIC_SEARCH_USERNAME=\"$ELASTIC_SEARCH_USERNAME\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json
            cat docker-compose.json|jq ".services.insight_server.environment.ELASTIC_SEARCH_PASSWORD=\"$ELASTIC_SEARCH_PASSWORD\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json
        else
            #Remove credentials from the url altogether
            cat docker-compose.json|jq 'del(.services.insight_server.environment.ELASTIC_SEARCH_PASSWORD)' > ~/tmpfile && mv ~/tmpfile docker-compose.json
            cat docker-compose.json|jq 'del(.services.insight_server.environment.ELASTIC_SEARCH_USERNAME)' > ~/tmpfile && mv ~/tmpfile docker-compose.json
        fi
    fi
        
}

externalizeMongoDB() {
    title "$MONGO_LABEL"
    log "Install $MONGO_LABEL as part of this installation? (Choose 'n' if you want to externalize $MONGO_LABEL)"
    getUserChoice "Install Mongo? [Y/n]" "y n Y N" "y"
    INSTALL_MONGO=${users_choice}
    addToEnvFile "INSTALL_MONGO"
    if [[ $INSTALL_MONGO =~ n|N ]]; then
        getMongoDetails
        seedMongo
        updateMongoDetails
    else
        removeFromEnvFile "MONGODB_HOST"
        removeFromEnvFile "MONGODB_PORT"
        removeFromEnvFile "MONGODB_ADMIN_USERNAME"
        removeFromEnvFile "MONGODB_ADMIN_PASSWORD"
    fi
}

getMongoDetails() {
    #Variables for Mongo
    : ${MONGODB_HOST:="docker.for.mac.localhost"}
    : ${MONGODB_PORT:="27017"}
    : ${MONGODB_ADMIN_USERNAME:="admin"}
    : ${MONGODB_ADMIN_PASSWORD:="password"}

    #Ask the host to connect to
    getUserInput "Please enter the $MONGO_LABEL Host" "$MONGODB_HOST"
    MONGODB_HOST=$users_choice
    addToEnvFile "MONGODB_HOST"

    #Ask the port to connect to
    getUserInput "Please enter the $MONGO_LABEL Port" "$MONGODB_PORT"
    MONGODB_PORT=$users_choice
    addToEnvFile "MONGODB_PORT"

    #Ask for admin login to create databases and users
    getUserInput "Please enter the $MONGO_LABEL admin username" "$MONGODB_ADMIN_USERNAME"
    MONGODB_ADMIN_USERNAME=$users_choice
    addToEnvFile "MONGODB_ADMIN_USERNAME"

    #Ask for admin password to create databases and users
    getUserInput "Please enter the $MONGO_LABEL admin password" "$MONGODB_ADMIN_PASSWORD"
    MONGODB_ADMIN_PASSWORD=$users_choice
    addToEnvFile "MONGODB_ADMIN_PASSWORD"

}

updateMongoDetails() {
    if [[ $INSTALL_MONGO =~ n|N ]]; then
        #Remove the mongo search container from the docker-compose file
        cat docker-compose.json|jq 'del(.services.mongodb)' > ~/tmpfile && mv ~/tmpfile docker-compose.json
        
        #Remove the entry that makes insight_server & server depend on mongo container
        cat docker-compose.json|jq 'del( .services.insight_server.depends_on[] | select( . == "mongodb") )'  > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq 'del( .services.scheduler.depends_on[] | select( . == "mongodb") )'  > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq 'del( .services.server.depends_on[] | select( . == "mongodb") )'  > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq 'del( .services.server.links[] | select( . == "mongodb") )'  > ~/tmpfile && mv ~/tmpfile docker-compose.json
        
        #remove the HOST entry and add the correct one
        cat docker-compose.json|jq 'del( .services.server.environment[] | select( test("SPRING_DATA_MONGODB_HOST") )  ) '  > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq '.services.server.environment += [ "SPRING_DATA_MONGODB_HOST='$MONGODB_HOST'" ]' > ~/tmpfile && mv ~/tmpfile docker-compose.json

        #remove the PORT entry and add the correct one
        cat docker-compose.json|jq 'del( .services.server.environment[] | select( test("SPRING_DATA_MONGODB_PORT") )  ) '  > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq '.services.server.environment += [ "SPRING_DATA_MONGODB_PORT='$MONGODB_PORT'" ]' > ~/tmpfile && mv ~/tmpfile docker-compose.json

        #add mongo details to insight server
        cat docker-compose.json|jq ".services.insight_server.environment.MONGO_URL=\"$MONGODB_HOST:$MONGODB_PORT\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq ".services.insight_server.environment.MONGODB_ADMIN_USERNAME=\"$MONGODB_ADMIN_USERNAME\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq ".services.insight_server.environment.MONGODB_ADMIN_PASSWORD=\"$MONGODB_ADMIN_PASSWORD\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json

        #add mongo details to scheduler
        cat docker-compose.json|jq ".services.scheduler.environment.MONGO_URL=\"$MONGODB_HOST:$MONGODB_PORT\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq ".services.scheduler.environment.MONGODB_ADMIN_USERNAME=\"$MONGODB_ADMIN_USERNAME\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq ".services.scheduler.environment.MONGODB_ADMIN_PASSWORD=\"$MONGODB_ADMIN_PASSWORD\"" > ~/tmpfile && mv ~/tmpfile docker-compose.json
    fi
}

seedMongo() {
    warn "Files which can be used to seed $MONGO_LABEL are now available at [$JFMC_MOUNT_ROOT/jfmc-installer/mongodb] \
Please seed $MONGO_LABEL before starting Mission-Control" #Don't add a tab to this line
}

MAX_WAIT_TIME=120
compareResponse() {
    local URL=$1
    local expectedOutput=$2
    local service=$3
    local targetNode=$4
    local curlOptions="-s --max-time 30"

    log "Checking $service status"
    local actualOutput=
    local curl_response
    # wait for service to be up, check every 2 secs
    # wait for a maximum of 50 seconds, so it doesn't get into an infinite loop
    WAIT_TIME=0
    while [[ "$actualOutput" != *"$expectedOutput"* ]] && [[ $WAIT_TIME -lt $MAX_WAIT_TIME ]] ; do
            echo -n "."
            sleep 2
            WAIT_TIME=$((WAIT_TIME + 3))
            curl_response=$(curl $curlOptions -H "Content-Type: application/json" "$URL" )  || true

            # check if this string contains a valid json
            if echo "$curl_response" | jq -e . &>/dev/null ; then
                actualOutput=$(echo "$curl_response" | jq "$targetNode" ) || true
            fi
            
            # if jq is not installed, assign the response for further processing
            # health check need not fail for this reason 
            hash jq 2>/dev/null || actualOutput="$curl_response"
    done

    if [[ "$actualOutput" == *"$expectedOutput"* ]]
    then
        echo -n " OK"
    else
        log "Actual Output: [$curl_response] of request [$URL] did not contain: [$expectedOutput] in node: [$targetNode]"
        exitOnError "$service is up but reporting issues. Please inspect the actual output and take necessary steps to fix the problem"
    fi
}

connectNetwork(){
    docker network connect jfmc_default jfmc_installer
}

healthCheck(){
    connectNetwork
    compareResponse "http://scheduler:8080/schedulerservice/api/status" "ACTIVE" "jfmc_scheduler_1" ".schedulerstatus"
    compareResponse "http://executor:8080/executorservice/api?action=status" "ACTIVE" "jfmc_executor_1" ".executorstatus"
    compareResponse "http://insight_server:80/api/status" 'SUCCESS' "jfmc_insight_server_1" ".request.status"
}

copySeedData() {
    mkdir -p $INSTALLER_DATA_FOLDER/mongodb
    cp mongodb/* $INSTALLER_DATA_FOLDER/mongodb
}

backupComposeFile() {
    if [ -f ${INSTALLER_DATA_FOLDER}/docker-compose.json ]; then
        cp ${INSTALLER_DATA_FOLDER}/docker-compose.json ${INSTALLER_DATA_FOLDER}/docker-compose.json.backup
        warn "The current docker-compose file has been backed up as docker-compose.json.backup. \n\
Once the upgrade is complete, you can reconcile any manual changes you may have made to the file"
    else
        cp docker-compose.json ${INSTALLER_DATA_FOLDER}/docker-compose.json
    fi
}

chooseTypeOfUpgrade(){
    log "Would you prefer a standard Upgrade? (Choose 'N' if you want to change any choices you made earlier)"

    getUserChoice "Perform a standard Upgrade? [Y/n]" "y n Y N" "y"
    STANDARD_UPGRADE=${users_choice}

    source "${JFMC_CONF}"
    if [[ $STANDARD_UPGRADE =~ y|Y ]]; then
        updateElasticSearchDetails
        updateMongoDetails
    else
        warn "$LOCALHOST_WARNING"
        externalizeElasticSearch
        externalizeMongoDB
    fi
    cp docker-compose.json ${INSTALLER_DATA_FOLDER}/docker-compose.json
}

chooseTypeOfInstallation(){
    title "Choose Type of Installation"

    log "Mission Control uses $ELASTIC_SEARCH_LABEL and $MONGO_LABEL. The installer can install these automatically. \n\
Would you prefer a standard installation? (If you choose 'N', the installer will ask you before installing any 3rd-party database)"

    getUserChoice "Perform a standard Installation? [Y/n]" "y n Y N" "y"
    STANDARD_INSTALLATION=${users_choice}

    addToEnvFile "STANDARD_INSTALLATION"
    if [[ $STANDARD_INSTALLATION =~ n|N ]]; then
        warn "$LOCALHOST_WARNING"
        externalizeElasticSearch
        externalizeMongoDB
    fi
    cp docker-compose.json ${INSTALLER_DATA_FOLDER}/docker-compose.json
}

# Move all installer files from user's home to data location in case of upgrades
# Keep only data location info in home location
migrateInstallInfo()
{
    if [[ ! -f "$INSTALLED_VERSION_FILE" && -f "$DEFAULT_INSTALLER_DATA_FOLDER/$INSTALLED_VERSION_FILE_NAME" ]]; then
        runDockerCommand stop \
        && mkdir -p "${INSTALLER_DATA_FOLDER}" \
        && cp -fr "$DEFAULT_INSTALLER_DATA_FOLDER"/* "${INSTALLER_DATA_FOLDER}"/ \
        && rm -fr "$DEFAULT_INSTALLER_DATA_FOLDER"/* \
        && cp "${INSTALLER_DATA_FOLDER}/$JFMC_CONF_FILE_NAME"  "${JFMC_CONF}" \
        && rm -f "${INSTALLER_DATA_FOLDER}/$JFMC_CONF_FILE_NAME" \
        && loadInstalledVersion \
        || exitOnError "Unable to migrate install information"
    fi
}

# Remove any ports exposed on host, reset exposed ports and add 8080 as the only exposed port    
removeExposedPorts() {

    #Don't remove exposed ports on dev machines
    if [ ! -z "$JFMC_DOCKER_REPO_LOGIN" ]; then
        return 0
    fi
    
    # Elasticsearch
    ( cat docker-compose.json | grep -wq '"elasticsearch"' ) || EXIT_CODE=$? && true
    if [[ -z $EXIT_CODE || $EXIT_CODE == 0 ]]; then
        #log "Removing exposed ports on $ELASTIC_SEARCH_LABEL"
        cat docker-compose.json|jq 'del( .services.elasticsearch.ports[] | select( . ) )' > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq 'del( .services.elasticsearch.expose[] | select( . ) )' > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq '.services.elasticsearch.expose += [ "9200", "9300" ]' > ~/tmpfile && mv ~/tmpfile docker-compose.json
    fi

    # MongoDB
    ( cat docker-compose.json | grep -wq '"mongodb"' ) || EXIT_CODE=$? && true
    if [[ -z $EXIT_CODE || $EXIT_CODE == 0 ]]; then
        #log "Removing exposed ports on $MONGO_LABEL"
        cat docker-compose.json|jq 'del( .services.mongodb.ports[] | select( . ) )' > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq 'del( .services.mongodb.expose[] | select( . ) )' > ~/tmpfile && mv ~/tmpfile docker-compose.json
        cat docker-compose.json|jq '.services.mongodb.expose += [ "27017","27018" ]' > ~/tmpfile && mv ~/tmpfile docker-compose.json
    fi

    # Scheduler
    #log "Removing exposed ports on Scheduler"
    cat docker-compose.json|jq 'del( .services.scheduler.ports[] | select( . ) )' > ~/tmpfile && mv ~/tmpfile docker-compose.json
    cat docker-compose.json|jq 'del( .services.scheduler.expose[] | select( . ) )' > ~/tmpfile && mv ~/tmpfile docker-compose.json
    cat docker-compose.json|jq '.services.scheduler.expose += [ "8080" ]' > ~/tmpfile && mv ~/tmpfile docker-compose.json

    # Executor
    #log "Removing exposed ports on Executor"
    cat docker-compose.json|jq 'del( .services.executor.ports[] | select( . ) )' > ~/tmpfile && mv ~/tmpfile docker-compose.json
    cat docker-compose.json|jq 'del( .services.executor.expose[] | select( . ) )' > ~/tmpfile && mv ~/tmpfile docker-compose.json
    cat docker-compose.json|jq '.services.executor.expose += [ "8080" ]' > ~/tmpfile && mv ~/tmpfile docker-compose.json
    
    # Insight-Server
    #log "Removing exposed ports on Insight-Serve"
    cat docker-compose.json|jq 'del( .services.insight_server.ports[] | select( . ) )' > ~/tmpfile && mv ~/tmpfile docker-compose.json
    cat docker-compose.json|jq 'del( .services.insight_server.expose[] | select( . ) )' > ~/tmpfile && mv ~/tmpfile docker-compose.json
    cat docker-compose.json|jq '.services.insight_server.expose += [ "80","443" ]' > ~/tmpfile && mv ~/tmpfile docker-compose.json

}

##################################
############## MAIN ##############
##################################

main() {
    aparse $@
    migrateInstallInfo
    isCommandAllowed ${command}

    case $command in
        installLocal)
            ( setVersions && \
              log "\nINFO: Mission Control was successfully installed. Run: ./mission-control start"
            ) || exitOnError "Installation failed"
            ;;
        install)
            DATE=$(date +"%Y%m%d%H%M")
            INSTALLATION_LOG_FILE="${INSTALLER_DATA_FOLDER}/installation.${DATE}.log"
            ( 
              copySeedData && \
              chooseTypeOfInstallation && \
              runDockerCommand pull && \
              genInternalCerts && \
              setVersions && \
              removeExposedPorts && \
              cp docker-compose.json ${INSTALLER_DATA_FOLDER}/docker-compose.json && \
              log "\nINFO: Mission Control was successfully installed. Run: ./mission-control start"
            ) | tee -a $INSTALLATION_LOG_FILE || exitOnError "Installation failed. Logs can be found at $DISPLAY_INSTALL_LOG_LOCATION"
            ;;
        upgrade)
            ( 
              backupComposeFile && \
              chooseTypeOfUpgrade && \
              runDockerCommand pull && \
              runDockerCommand stopApp && \
              backwardCmpatibleProcess && \
              setVersions && \
              removeExposedPorts && \
              cp docker-compose.json ${INSTALLER_DATA_FOLDER}/docker-compose.json && \
              log "\nINFO: Mission Control was successfully upgraded. Run: ./mission-control start" \
            ) || exitOnError "Upgrade failed"
            ;;
        upgradeLocal)
            ( runDockerCommand stopApp && \
              setVersions && \
              log "\nINFO: Mission Control was successfully upgraded. Run: ./mission-control start" \
            ) || exitOnError "Upgrade failed"
            ;;
        generateCerts)
            ( genInternalCerts && \
              log "\nINFO: Mission Control internal certs were successfully configured." \
            ) || exitOnError "Generating certificates failed"
            ;;
        start)
            runDockerCommand "up -d --remove-orphans"
            createUsers
            healthCheck
            log "\nINFO: Mission Control was successfully started and can be accessed on port $JFMC_PORT"
            ;;
        ps|logs|stop|kill|restart)
            runDockerCommand ${command}
            ;;
        version)
            if [ -f $INSTALLED_VERSION_FILE ]; then
                echo "Mission Control version: ${INSTALLED_VERSION}"
            else
                echo "Mission Control is not installed"
            fi
            ;;
        x-compose)
                xComposeCmd="docker-compose -f docker-compose.json -p ${COMPOSE_PROJECT_NAME} ${ALL_ARGS}"
                echo "Advanced user: Going to execute '${xComposeCmd}'"
                ${xComposeCmd}
            ;;
        info)
               echo "Mission Control info:"
               echo "project name: ${COMPOSE_PROJECT_NAME}"
               echo "docker repo: ${JFMC_DOCKER_REPO}"
               echo "docker mount folder: ${JFMC_MOUNT_ROOT}"
            ;;
        healthCheck)
                healthCheck
            ;;
        *)
            usage
            ;;
    esac
}

main $@
