#!/bin/bash

OFFLINE_MODE=false
AUTOMATION_MODE="false"
JFI_INTERACTIVE=false

# To pull images from a registry which needs login credentials, JFMC_DOCKER_REPO_LOGIN and JFMC_DOCKER_REPO_PASSWORD needs to be set as environment variables
#
# export JFMC_DOCKER_REPO_LOGIN=<username>
# export JFMC_DOCKER_REPO_PASSWORD=<token>
# 
# The lines above can be used to set env variables for this session. Add to ~/.bashrc to set them permanently
# 
# Usage : ./mission-control.sh install
# Additionally, passing -i will allow the user to choose which version to download interactively

while [[ $# > 1 ]]; do
    case "$1" in
        -u)
            [[ "$JFMC_DOCKER_REPO_LOGIN" ]] || credsChangeMessage
            shift 2
        ;;
        -p)
            [[ "$JFMC_DOCKER_REPO_PASSWORD" ]] || credsChangeMessage
            shift 2
        ;;
        -a | --use-defaults)
            echo "Running in non-interactive mode"
            AUTOMATION_MODE="true"
            shift
        ;;
        -o | --offline)
            echo "Running in offline mode"
            OFFLINE_MODE=true
            shift
        ;;
        -i | --custom)
            JFI_INTERACTIVE=true
            shift
        ;;
        * )
            CMD="$CMD $1"
            shift
        ;;
    esac
done

[[ $CMD ]] || CMD="$@"

setDevVersions(){
    # A pattern can be entered to fetch latest images. For example, tag pattern with value : *master* will get latest image which contains master as part of it ( or image built from master branch )
    # NOTE : please enter complete tag if you do not wish to fetch latest images for every upgrade
    JFI_INSIGHT_SERVER_VERSION="*master*"
    JFI_SCHEDULER_VERSION="*master*"
    JFI_EXECUTOR_VERSION="*master*"
    #In interactive mode, ask for the version, & if nothing is entered, take the original value
    if [ "${JFI_INTERACTIVE}" == true ]; then
        echo -e "A pattern can be entered to fetch latest images. For example, tag pattern with value : *master* will get latest image which contains master as part of it ( or image built from master branch )"
        echo -e "NOTE : please use full tag if you do not wish to fetch latest images for every upgrade"
        
        local temp_var=${JFI_SCHEDULER_VERSION}
        read -p "Enter scheduler version [${JFI_SCHEDULER_VERSION}]: " JFI_SCHEDULER_VERSION
        : ${JFI_SCHEDULER_VERSION:=$temp_var}

        local temp_var=${JFI_EXECUTOR_VERSION}
        read -p "Enter executor version [${JFI_EXECUTOR_VERSION}]: " JFI_EXECUTOR_VERSION
        : ${JFI_EXECUTOR_VERSION:=$temp_var}

        local temp_var=${JFI_INSIGHT_SERVER_VERSION}
        read -p "Enter insight server version [${JFI_INSIGHT_SERVER_VERSION}]: " JFI_INSIGHT_SERVER_VERSION
        : ${JFI_INSIGHT_SERVER_VERSION:=$temp_var}
    fi
}

getLatestTag() {
    local tag=${1:-"2*"}
    local repoPath=${2:-"jfrog"}
    local repo=${3:-"insight-docker"}
    local devRepoURL=${4:-"https://repo.jfrog.io/artifactory"}
    local response

    if [[ "${JFMC_DOCKER_REPO}" = *"bangalore"* ]]; then
        devRepoURL="https://bangalore.jfrog.io/bangalore"
        repo="jfrog-docker"
    fi

    response=$(curl -s -XPOST -u${JFMC_DOCKER_REPO_LOGIN}:${JFMC_DOCKER_REPO_PASSWORD} ${devRepoURL}/api/search/aql -d "items.find({\"repo\":\""$repo"\", \"type\":\"folder\", \"path\":{\"\$match\":\""$repoPath"*\"}, \"name\":{\"\$match\":\""$tag"\"}}).sort({\"\$desc\" : [\"created\"]}).limit(1)" -H "Content-Type: text/plain"| jq -r ".results[].name") || true

    if [[ -z "$response" || "$response" = "" || "$response" = true || "$response" = null ]]; then
        echo ""
    else
        echo "$response"
    fi
}

validateTag(){
    local tagName=$1
    local tagResponse=$2
    local tagPattern=$3

    if [[ -z "${tagResponse}" || "${tagResponse}" = "" ]]; then
        echo -e "\nFailed to fetch latest modified tag for ${tagName} with following value : [ ${tagPattern} ], please check if this is available in repo"
        exit 1
    fi
}

setLatestTags(){
    local tmpVersion
    local currentVersionEnv="${JFMC_DIR}/current_versions"
    local serviceName
    local serviceKey

    # Check if jq is installed
    hash jq 2>/dev/null || { echo >&2 "Please install jq to continue, this is used to parse json response from api"; exit 1; }
    local services=(scheduler=JFI_SCHEDULER_VERSION executor=JFI_EXECUTOR_VERSION server=JFI_INSIGHT_SERVER_VERSION)
    local serviceList=""
    for service in "${services[@]}"
    do
        serviceName=$(echo $service | awk -F '=' '{print $1}')
        serviceKey=$(echo $service | awk -F '=' '{print $2}')

        # Get the value associated with this variable (indirect reference)
        serviceValue=${!serviceKey}

        tmpVersion=$(getLatestTag "$serviceValue" "jfrog/insight-$serviceName")
        validateTag "$serviceKey" "$tmpVersion" "$serviceValue"
        export $serviceKey="$tmpVersion"

        serviceList="$serviceList $serviceKey"
    done

    [[ "$CMD" == *"start"* &&  ! -z "$currentVersionEnv" && "$currentVersionEnv" != "" && -f "$currentVersionEnv" ]] && source "$currentVersionEnv" || true

    if [[ "$CMD" == *"install"* || "$CMD" == *"upgrade"* || "$CMD" == *"start"* ]]; then
        echo -e "Scheduler : ${JFI_SCHEDULER_VERSION}"
        echo -e "Executor : ${JFI_EXECUTOR_VERSION}"
        echo -e "Insight Server : ${JFI_INSIGHT_SERVER_VERSION}\n"
        rm -f "$currentVersionEnv";
        addToEnvFile "$serviceList" "$currentVersionEnv";
    fi
}

# Store microservice versions for future reference
# To handle a scenario where upgrade got one version and start recieved a newer one
addToEnvFile(){
    if [[ -z "$1" || "$1" == "" || -z "$2" || "$2" == "" ]]; then
        return
    fi
    local env_list=($1)
    local env=
    local currentVersionEnv=$2

    for env_key in "${env_list[@]}"
    do
        # Get the value associated with this variable (indirect reference)
        env_value=${!env_key}
        if [[ ! -z "$env_value" && "$env_value" != "" ]]; then
            [[ ! -z "$currentVersionEnv" && "$currentVersionEnv" != "" && ! -f "$currentVersionEnv" ]] && touch "$currentVersionEnv" || true
            echo "export $env_key=\"$env_value\"" >> "$currentVersionEnv"
        fi
    done
}

isNotMilestone(){
    if [[ "$JFMC_VERSION" = *"m00"* ]]; then
        echo ""
    else
        echo "no its not, an opportunity to play with latest images"
    fi
}

##################################
############## MAIN ##############
##################################

setUp
[[ $(isNotMilestone) ]] && setDevVersions || true
createInstallerConfigFile
loadInstallerConfigFile
setupDockerParameters
setUpDevDockerParamters
[[ $(isNotMilestone) ]] && setLatestTags || true
setCommonEnvVars

exec docker run --name "jfmc_installer" --rm \
${COMMON_ENV_VARS} ${IMAGE} "${CMD}"

exit $?
