#!/bin/bash

MONGO_LABEL='MongoDB'

#Print the input with additional formatting to indicate a section/title
title () {
    echo
    echo "-----------------------------------------------------"
    printf "| %-50s|\n" "$1"
    echo "-----------------------------------------------------"
}

# This function prints the echo with color.
#If invoked with a single string, treats it as INFO level
#Valid inputs for the second parameter are DEBUG and INFO
log() {
    echo ""
    echo -e $1
    echo ""
}

# Utility method to display warnings and important information
warn() {
    echo ""
    echo -e "\033[33m $1 \033[0m"
    echo ""
}

errorExit() {
    echo; echo -e "\033[31mERROR:$1\033[0m"; echo
    exit 1
}

INSTALLER_DIR="$( cd "$( dirname $0 )/" && pwd )"

attempt_number=0
mongoIsNotReady() {
	mongo $MONGO_PORT_OPTION --version > /dev/null 2>&1
    outcome1=$?
    ss -lntu | grep -wq $MONGODB_PORT 2>&1
    outcome2=$?
    if [[ $outcome1 -eq 0 ]] && [[ $outcome2 -eq 0  ]]; then
        return 0
    else
        if [ $attempt_number -gt 10 ]; then
            errorExit "Unable to proceed. $MONGO_LABEL is not reachable. This can occur if the service is not running \
or the port is not accepting requests at $MONGODB_PORT. Gave up after $attempt_number attempts"
        fi
        let "attempt_number=attempt_number+1"
        return 1
    fi
}

createMongoData() {
    # Uncomment only if required - check why this was added
    #If this code has already been run once, take the original file
    # if [ ! -f $INSTALLER_DIR/createMongoUsers.js.backup ]; then
    #     cp $INSTALLER_DIR/createMongoUsers.js $INSTALLER_DIR/createMongoUsers.js.backup
    # else 
    #     cp $INSTALLER_DIR/createMongoUsers.js.backup $INSTALLER_DIR/createMongoUsers.js
    # fi

    log "Waiting for $MONGO_LABEL to get ready using the commands: \"mongo $MONGO_PORT_OPTION --version\" & \"ss -lntu | grep -wq $MONGODB_PORT\""
    while ! mongoIsNotReady
    do
            sleep 5
            echo -n '.'
    done
    log "$MONGO_LABEL is ready. Executing commands"

    ## Find out the credentials for Mongo
    local ADMIN_USER="admin"
    local ADMIN_PWD="password"
    local EMPTY_MONGO="yes"

    if [ ! -z $LOCAL_MONGO ]; then
        EMPTY_MONGO="no"
        choice="na"
        default_choice="y"
        until [[ "${choice}" == "y" || "${choice}" == "n" || "${choice}" == "Y" || "${choice}" == "N" ]]; do
            echo;echo;
            sleep 1 #This ensures correct placement of the question.
            read -p "Is this a fresh installation with no users? (If you choose 'y', the installer will create an admin user) [Y/n]: " choice
            : ${choice:=$default_choice}
        done
        if [[ $choice == "y" || $choice == "Y" ]]; then
            EMPTY_MONGO="yes"
        fi
        if [ $LOCAL_MONGO == "yes" ]; then
            if [[ $MONGODB_ADMIN_USERNAME ]]; then
                ADMIN_USER=$MONGODB_ADMIN_USERNAME
            else
                echo; echo;
                sleep 1 #This ensures correct placement of the question.
                read -p "Please enter the $MONGO_LABEL admin user ID [$ADMIN_USER] : " choice
                : ${choice:=$ADMIN_USER}
                ADMIN_USER=$choice
            fi
            if [[ $MONGODB_ADMIN_PASSWORD ]]; then
                ADMIN_PWD=$MONGODB_ADMIN_PASSWORD
            else
                echo; echo;
                sleep 1 #This ensures correct placement of the question.
                read -p "Please enter the $MONGO_LABEL admin user password [$ADMIN_PWD] : " choice
                : ${choice:=$ADMIN_PWD}
                ADMIN_PWD=$choice
            fi
        fi
    fi

    sed -i "s/__EMPTY_DB__/$EMPTY_MONGO/g" $INSTALLER_DIR/createMongoUsers.js
    sed -i "s/__ADMIN_USER__/$ADMIN_USER/g" $INSTALLER_DIR/createMongoUsers.js
    sed -i "s/__ADMIN_USER_PWD__/$ADMIN_PWD/g" $INSTALLER_DIR/createMongoUsers.js
    
    mongo $MONGO_PORT_OPTION admin $INSTALLER_DIR/createMongoUsers.js > /dev/null 2>&1

    EXIT_CODE="$?"
    if [[ $EXIT_CODE -eq 0 ]]; then
        log "$MONGO_LABEL setup is now complete"
    fi
    exit $EXIT_CODE
}

main() {
    if [[ -z $MONGODB_PORT ]]; then
        title "$MONGO_LABEL Manual Setup"
        log "This script will attempt to seed $MONGO_LABEL with the databases and users needed by JFrog Mission Control"
        warn "Please enter the same details as you entered during installation. If the details are incorrect, you may need to rerun the installation"
        LOCAL_MONGO="yes" #IF port is not available, this is most likely an externalized database and the user is seeding manually
        echo; echo;
        local DEFAULT_PORT="27017"
        read -p  "Please enter the $MONGO_LABEL port [$DEFAULT_PORT]:" choice
        : ${choice:=$DEFAULT_PORT}
        MONGODB_PORT=$choice
    fi

    [[ ! -z "$MONGODB_PORT" && "$MONGODB_PORT" != "" ]] && export MONGO_PORT_OPTION="--port $MONGODB_PORT" || export MONGO_PORT_OPTION=""
    
    createMongoData
}

main

